unit TNWObj;

interface

uses Classes,
        TNWBind;

type
    TNWObject = class
    private
        { Private-dclarations }
        FProprietaire: TNWBindery;
        FObjID: LongInt;
        FObjNom: string;
        FObjType: Word;
        FObjFlags: Byte;
        FObjSecurite: Byte;
        FObjTempsConnexion: LongInt;
        FObjNbBlockAlloue: LongInt;
        FObjRestrictionDisk: LongInt;
        procedure SetObjNom(NouveauNom: String);
        procedure SetObjSecurite(NouvelleSecurite: Byte);
        procedure FreeObjListe(Liste: TStrings);
    protected
        { Protected-dclarations }
        DernierAliasNomPpte: string;
        procedure ChercheObjectsDansSet(Objects: TStrings; PpteNom: String);
        procedure AssignObjectsDansSet(PpteNom: String; Objects: TStrings);
        procedure ChercheInfosDans(Infos: TStrings; PpteNom: String);
        procedure AssignInfosDans(PpteNom: String; Infos: TStrings);
        function ChercheInfoDans(PpteNom: String): String;
        procedure AssignInfoDans(PpteNom: String; Info: String);
    public
        { Public-dclarations }
        constructor Create(Parent: TNWBindery; ObjID: LongInt; ObjNom: String;
                            ObjType: Word; ObjHasPptes, ObjFlag, ObjSecurite: Byte); virtual;
        function Delete: Word;
        function VerifieMotDePasse(MotDePasse: String): Word;
        function ChangeMotDePasse(AncienMotDePasse, NouveauMotDePasse: String): Word;
        function DesactiveMotDePasse(MotDePasse: String): Word;
        procedure CherchePropriete(Liste: Tstrings; AliasNom: String);
        function CreerPropriete(PpteNom: string; PpteFlags, PpteSecurite: Byte): Word;
        procedure ChercheDescription(Descriptions: TStrings);
        procedure AssignDescription(Descriptions: TStrings);
        procedure ChercheLocations(Locations: TStrings);
        procedure AssignLocations(Locations: TStrings);
        procedure ChercheDepartements(Departements: TStrings);
        procedure AssignDepartements(Departements: TStrings);
        procedure ChercheOrganisations(Organisations: TStrings);
        procedure AssignOrganisations(Organisations: TStrings);
        procedure ChercheToutObjects(ToutObjects: TStrings);
        procedure AssignToutObjects(ToutObjects: TStrings);
        property Proprietaire: TNWBindery read FProprietaire;
        property ObjID: LongInt read FObjID;
        property ObjNom: string read FObjNom write SetObjNom;
        property ObjType: Word read FObjType;
        property ObjFlags: Byte read FObjFlags;
        property ObjSecurite: Byte read FObjSecurite write SetObjSecurite;
        property ObjTempsConnexion: LongInt read FObjTempsConnexion;
        property ObjNbBlockAlloue: LongInt read FObjNbBlockAlloue;
        property ObjRestrictionDisk: LongInt read FObjRestrictionDisk;
    end;

implementation

uses SysUtils,
        NWConst,NWConst2, NWCalls,
        TNWPpte, TNWGroup, TNWUti, TNWSvrPr, TNWSvrFi, TNWFilPr, TNWProf, NLMUti;

{************************* P U B L I C **************************************************}
constructor TNWObject.Create(Parent: TNWBindery; ObjID: LongInt; ObjNom: String;
                    ObjType: Word; ObjHasPptes, ObjFlag, ObjSecurite: Byte);
begin
    FProprietaire := Parent;
    FObjID := ObjID;
    FObjNom := ObjNom;
    FObjType := ObjType;
    FObjFlags := ObjFlags;
    FObjSecurite := ObjSecurite;
    NWGetObjectDiskSpaceLeft(Proprietaire.NWConn.Handle, ObjID, @FObjTempsConnexion, @FObjNbBlockAlloue, @FObjRestrictionDisk);
end;


function TNWObject.Delete: Word;
var PObjNom: PChar;
begin
    GetMem(PObjNom, 48);
    StrPCopy(PObjNom, FObjNom);
    Result := DeleteObject(FProprietaire.NWConn.Handle, PObjNom, FObjType);
    FreeMem(PObjNom, 48);
end;

{*************************************************************************************}
function TNWObject.VerifieMotDePasse(MotDePasse: String): Word;
var PMotDePasse, PObjNom: Pchar;
begin
    GetMem(PMotDePasse, 255);
    StrPCopy(PMotDePasse, MotDePasse);
    GetMem(PObjNom, 48);
    StrPCopy(PObjNom, FObjNom);
    Result := VerifyObjectPassword(Proprietaire.NWConn.Handle, PObjNom, FObjType, PMotDePasse);
    FreeMem(PObjNom, 48);
    FreeMem(PMotDePasse, 255);
end;


function TNWObject.ChangeMotDePasse(AncienMotDePasse, NouveauMotDePasse: String): Word;
var PAncienMotDePasse, PNouveauMotDePasse, PObjNom: PChar;
begin
    GetMem(PAncienMotDePasse, 255);
    StrPCopy(PAncienMotDePasse, AncienMotDePasse);
    GetMem(PNouveauMotDePasse, 255);
    StrPCopy(PNouveauMotDePasse, NouveauMotDePasse);
    GetMem(PObjNom, 48);
    StrPCopy(PObjNom, FObjNom);
    Result := ChangeObjectPassword(Proprietaire.NWConn.Handle, PObjNom, FObjType, PAncienMotDePasse, PNouveauMotDePasse);
    FreeMem(PObjNom, 48);
    FreeMem(PNouveauMotDePasse, 255);
    FreeMem(PAncienMotDePasse, 255);
end;


function TNWObject.DesactiveMotDePasse(MotDePasse: String): Word;
var PMotDePasse, PObjNom: Pchar;
begin
    GetMem(PMotDePasse, 255);
    StrPCopy(PMotDePasse, MotDePasse);
    GetMem(PObjNom, 48);
    StrPCopy(PObjNom, FObjNom);
    Result := DisallowObjectPassword(Proprietaire.NWConn.Handle, PObjNom, FObjType, PMotDePasse);
    FreeMem(PObjNom, 48);
    FreeMem(PMotDePasse, 255);
end;


{*************************************************************************************}
procedure TNWObject.CherchePropriete(Liste: TStrings; AliasNom: String);
var PObjNom, PPpteNomCherche, PPpteNomTrouve: Pchar;
    NumSequence: LongInt;
    PpteFlags, PpteSecurite, PpteADesValeurs , Encore: Byte;
    NouvellePpte: TNWPropriete;
begin
    FreeObjListe(Liste);
    Liste.Clear;
    DernierAliasNomPpte := AliasNom;
    GetMem(PObjNom, 48);
    StrPCopy(PObjNom, FObjNom);
    GetMem(PPpteNomCherche, 255);
    StrPCopy(PPpteNomCherche, AliasNom);
    GetMem(PPpteNomTrouve, 15);
    NumSequence := -1;
    While(ScanProperty(Proprietaire.NWConn.Handle, PObjNom, FObjType, PPpteNomCherche, @NumSequence, PPpteNomTrouve,
                        @PpteFlags, @PpteSecurite, @PpteADesValeurs, @Encore) = 0) do
    begin
        NouvellePpte := TNWPropriete.Create(Self, StrPas(PPpteNomTrouve), PpteFlags, PpteSecurite, PpteADesValeurs);
        Liste.AddObject(StrPas(PPpteNomTrouve), NouvellePpte);
    end;
    FreeMem(PPpteNomTrouve, 15);
    FreeMem(PPpteNomCherche,255);
    FreeMem(PObjNom, 48);
end;


function TNWObject.CreerPropriete(PpteNom: string; PpteFlags, PpteSecurite: Byte): Word;
var PObjNom, PPpteNom: Pchar;
begin
    GetMem(PObjNom, 48);
    StrPCopy(PObjNom, FObjNom);
    GetMem(PPpteNom, 256);
    StrPCopy(PPpteNom, PpteNom);
    Result := CreateProperty(Proprietaire.NWConn.Handle, PObjNom, ObjType, PPpteNom, PpteFlags, PpteSecurite);
    FreeMem(PPpteNom, 256);
    FreeMem(PObjNom, 48);
end;

procedure TNWObject.ChercheDescription(Descriptions: TStrings);
var Description: String[128];
begin
    Description := ChercheInfoDans('DESCRIPTION');
    if (Description[0] = #128) then Descriptions.SetText(@Description[1])
    else Descriptions.Clear;
end;

procedure TNWObject.AssignDescription(Descriptions: TStrings);
var Description: String[128];
    PDescription: PChar;
    Compteur: Integer;
begin
    PDescription := Descriptions.GetText;
    move(PDescription[0], Description[1], 127);
    Description[128] := #0;
    Description[0] := #128;
    AssignInfoDans('DESCRIPTION', Description);
end;


procedure TNWObject.ChercheLocations(Locations: TStrings);
begin
    ChercheInfosDans(Locations, 'L');
end;

procedure TNWObject.AssignLocations(Locations: TStrings);
begin
    AssignInfosDans('L', Locations);
end;


procedure TNWObject.ChercheDepartements(Departements: TStrings);
begin
    ChercheInfosDans(Departements, 'OU');
end;

procedure TNWObject.AssignDepartements(Departements: TStrings);
begin
    AssignInfosDans('OU', Departements);
end;

procedure TNWObject.ChercheOrganisations(Organisations: TStrings);
begin
    ChercheInfosDans(Organisations, 'O');
end;

procedure TNWObject.AssignOrganisations(Organisations: TStrings);
begin
    AssignInfosDans('O', Organisations);
end;

procedure TNWObject.ChercheToutObjects(ToutObjects: TStrings);
begin
    ChercheObjectsDansSet(ToutObjects, 'SEE_ALSO');
end;

procedure TNWObject.AssignToutObjects(ToutObjects: TStrings);
begin
    AssignObjectsDansSet('SEE_ALSO', ToutObjects);
end;


{********************************* P R I V A T E ************************************}
procedure TNWObject.SetObjNom(NouveauNom: String);
var PAncienNom, PNouveauNom: PChar;
    Ret: Word;
begin
    GetMem(PAncienNom, 48);
    StrPCopy(PAncienNom, FObjNom);
    GetMem(PNouveauNom, 48);
    StrPCopy(PNouveauNom, NouveauNom);
    Ret := RenameObject(Proprietaire.NWConn.Handle, PAncienNom, PNouveauNom, FObjType);
    if (Ret = 0) then FObjNom := NouveauNom;
    FreeMem(PAncienNom, 48);
    FreeMem(PNouveauNom, 48);
end;


procedure TNWObject.SetObjSecurite(NouvelleSecurite: Byte);
var Ret: Word;
    PObjNom: PChar;
begin
    GetMem(PObjNom, 48);
    StrPCopy(PObjNom, FObjNom);
    Ret := ChangeObjectSecurity(Proprietaire.NWConn.Handle, PObjNom, FObjType, NouvelleSecurite);
    if (Ret = 0) then FObjSecurite := NouvelleSecurite;
    FreeMem(PObjNom, 48);
end;


procedure TNWObject.ChercheObjectsDansSet(Objects: TStrings; PpteNom: String);
var PpteObjects: TNWPropriete;
    Compteur1, Compteur2, Ret: Integer;
    Buffer: String[128];
    NouveauObject: TNWObject;
    NumIDObj: LongInt;
    PNomObj: PChar;
    NomObj: String[47];
    TypeObj: Word;
begin
    FreeObjListe(Objects);
    Objects.Clear;
    GetMem(PNomObj, 48);
    PpteObjects := TNWPropriete.Create(Self, PpteNom, 0, 0, 0);
    Compteur1 := 1;
    repeat
        Buffer := PpteObjects.PpteValeurs[Compteur1];
        if (Buffer[0] = #128) then
        begin
            Compteur2 := 1;
            repeat
                NumIDObj := Byte(Buffer[Compteur2+3]);
                NumIDObj := NumIdObj shl 8 + Byte(Buffer[Compteur2+2]);
                NumIDObj := NumIDObj shl 8 + Byte(Buffer[Compteur2+1]);
                NumIDObj := NumIDObj shl 8+ Byte(Buffer[Compteur2]);
                if (NumIDObj <> 0) then
                begin
                    Ret := GetObjectName(Proprietaire.NWConn.Handle, NumIDObj, PNomObj, @TypeObj);
                    if (Ret = 0) then
                    begin
                        NomObj := StrPas(PNomObj);
                        case TypeObj of
                            OT_USER: NouveauObject := TNWUtilisateur.Create(
                                                    Proprietaire, NumIDObj, NomObj, TypeObj, 0, 0, 0);
                            OT_USER_GROUP: NouveauObject := TNWGroupe.Create(
                                                    Proprietaire, NumIDObj, NomObj, TypeObj, 0, 0, 0);
                            OT_PRINT_SERVER: NouveauObject := TNWServeurImprimante.Create(
                                                    Proprietaire, NumIDObj, NomObj, TypeObj, 0, 0, 0);
                            OT_FILE_SERVER: NouveauObject := TNWServeurFichier.Create(
                                                    Proprietaire, NumIDObj, NomObj, TypeObj, 0, 0, 0);
                            OT_PRINT_QUEUE: NouveauObject := TNWFileImpression.Create(
                                                    Proprietaire, NumIDObj, NomObj, TypeObj, 0, 0, 0);
                            $3501: NouveauObject := TNWProfile.Create(
                                                    Proprietaire, NumIDObj, NomObj, TypeObj, 0, 0, 0);
                            else NouveauObject := TNWObject.Create(
                                                    Proprietaire, NumIDObj, NomObj, TypeObj, 0, 0, 0);
                        end;
                        Objects.AddObject(NomObj, NouveauObject);
                    end;
                end;
                Compteur2 := Compteur2 + 4;
            until (Compteur2 > 128) or (NumIDObj = 0);
        end;
        Inc(Compteur1);
    until (Buffer[0] <> #128) or (NumIDObj = 0);
    FreeMem(PNomObj, 48);
    PpteObjects.Free;
end;


procedure TNWObject.AssignObjectsDansSet(PpteNom: String; Objects: TStrings);
var Compteur: Integer;
    PpteObjects: TNWPropriete;
begin
    PpteObjects := TNWPropriete.Create(Self, PpteNom, 0, 0, 0);
    PpteObjects.Delete;
    if Objects.Count > 0 then
    begin
        CreerPropriete(PpteNom, $02, $33);
        for Compteur := 0 to Objects.Count-1 do
            PpteObjects.AjouteObjectDansSet(TNWObject(Objects.Objects[Compteur]));
    end;
    PpteObjects.Free;
end;


procedure TNWObject.ChercheInfosDans(Infos: TStrings; PpteNom: String);
var PpteInfos: TNWPropriete;
    Info: string[128];
    PInfo: PChar;
    Compteur: Integer;
begin
    FreeObjListe(Infos);
    Infos.Clear;
    GetMem(PInfo, 129);
    PpteInfos := TNWPropriete.Create(Self, PpteNom, 0, 0, 0);
    Compteur := 1;
    repeat
        Info := PpteInfos.PpteValeurs[Compteur];
        StrPCopy(PInfo, Info);
        Info := StrPas(PInfo);
        Inc(Compteur);
        if (Info <> '') then Infos.Add(Info);
    until (Info = '');
    PpteInfos.Free;
    FreeMem(PInfo, 129);
end;

procedure TNWObject.AssignInfosDans(PpteNom: String; Infos: TStrings);
var Compteur: Integer;
    PpteInfos: TNWPropriete;
begin
    PpteInfos := TNWPropriete.Create(Self, PpteNom, 0, 0, 0);
    PpteInfos.Delete;
    if Infos.Count > 0 then
    begin
        CreerPropriete(PpteNom, $00, $33);
        for Compteur := 0 to Infos.Count-1 do
            PpteInfos.PpteValeurs[Compteur+1] := Infos.Strings[Compteur];
    end;
    PpteInfos.Free;
end;


function TNWObject.ChercheInfoDans(PpteNom: String): String;
var PpteInfo: TNWPropriete;
begin
    PpteInfo := TNWPropriete.Create(Self, PpteNom, 0, 0, 0);
    Result := PpteInfo.PpteValeurs[1];
    PpteInfo.Free;
end;


procedure TNWObject.AssignInfoDans(PpteNom: String; Info: String);
var PpteInfo: TNWPropriete;
begin
    PpteInfo := TNWPropriete.Create(Self, PpteNom, 0, 0, 0);
    PpteInfo.Delete;
    if (Info[0] <> #0) then
    begin
        CreerPropriete(PpteNom, $00, $32);
        PpteInfo.PpteValeurs[1] := Info;
    end;
    PpteInfo.Free;
end;

procedure TNWObject.FreeObjListe(Liste: TStrings);
var Compteur: Integer;
begin
    for Compteur := 0 to Liste.Count-1 do (Liste.Objects[Compteur] as TObject).Free;
    Liste.Clear;
end;

end.
